package com.compdfkit.pdfviewer.activities;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;
import androidx.core.view.MenuProvider;
import androidx.core.view.ViewCompat;
import androidx.core.view.WindowInsetsControllerCompat;

import android.graphics.Bitmap;
import android.graphics.RectF;
import android.graphics.Typeface;
import android.net.Uri;
import android.os.Bundle;
import android.util.Log;
import android.util.TypedValue;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.ImageView;
import android.widget.Switch;
import android.widget.TextView;
import android.widget.Toast;

import com.compdfkit.core.common.CPDFDocumentException;
import com.compdfkit.core.document.CPDFBates;
import com.compdfkit.core.document.CPDFDocument;
import com.compdfkit.core.document.CPDFHeaderFooter;
import com.compdfkit.core.page.CPDFPage;
import com.compdfkit.pdfviewer.R;
import com.compdfkit.pdfviewer.fragments.HeaderFooterConsoleFragment;
import com.compdfkit.pdfviewer.fragments.WatermarkConsoleFragment;
import com.compdfkit.pdfviewer.tools.ConvertMapper;
import com.compdfkit.pdfviewer.tools.DialogManager;
import com.compdfkit.pdfviewer.tools.DocumentHandle;
import com.compdfkit.pdfviewer.tools.HeaderFooterHandle;
import com.compdfkit.pdfviewer.tools.TextGenerator;
import com.google.android.material.tabs.TabLayout;

import java.io.File;
import java.util.Random;

public class HeaderFooterActivity extends AppCompatActivity implements MenuProvider {

    private int type = 0;
    private int currentHeaderFooterIndex = 0;
    private int pageCount = 0;

    private HeaderFooterConsoleFragment consoleFragment;

    private TabLayout tabLayout;
    private View pageContainer;
    private ImageView pageImage;

    private TextView[] headerFooterViews = new TextView[6];

    private int currentPageHeight = 0;
    private int currentPageWidth = 0;

    private int imageHeight;
    private int imageWidth;

    private CPDFDocument document = new CPDFDocument(this);

    private float sizeConvertRate = 0;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        int[] themes = new int[] {R.style.AppTheme, R.style.AppRedTheme, R.style.AppOrangeTheme, R.style.AppGreenTheme, R.style.AppCyanBlueTheme};
        int randomTheme = new Random().nextInt(5);
        setTheme(themes[randomTheme]);

        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_header_footer);
        addMenuProvider(this);

        Toolbar toolbar = findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        tabLayout = findViewById(R.id.tab_layout);
        initTabLayout();

        consoleFragment = (HeaderFooterConsoleFragment) getSupportFragmentManager().findFragmentByTag("console_fragment");

        File PDFFile = new File("/storage/emulated/0/TestPDFs/test.pdf");
        Uri PDFUri = Uri.fromFile(PDFFile);

        pageCount = DocumentHandle.reloadDocument(PDFFile, PDFUri, document);
        HeaderFooterHandle.setSettingPages(0, "0-" + (pageCount - 1));
        HeaderFooterHandle.setSettingPages(1, "0-" + (pageCount - 1));

        pageContainer = findViewById(R.id.page_container);

        pageImage = findViewById(R.id.page_image);
        pageImage.post(this::initPageImage);

        if (this.getResources().getConfiguration().uiMode == 0x11) {
            WindowInsetsControllerCompat wic = ViewCompat.getWindowInsetsController(getWindow().getDecorView());
            if (wic != null)
                wic.setAppearanceLightStatusBars(true);
        }

        initHeaderFooterViews();

        Log.d("TestBatesString", TextGenerator.generateBatesText("NB", "BN", 3, 20));
    }

    @Override
    public void onCreateMenu(@NonNull Menu menu, @NonNull MenuInflater menuInflater) {
        menuInflater.inflate(R.menu.menu_header_footer, menu);
    }

    @Override
    public boolean onMenuItemSelected(@NonNull MenuItem menuItem) {
        return false;
    }

    private void initTabLayout() {
        tabLayout.addOnTabSelectedListener(new TabLayout.OnTabSelectedListener() {
            @Override
            public void onTabSelected(TabLayout.Tab tab) {
                currentHeaderFooterIndex = tab.getPosition();
                consoleFragment.resetContentText(currentHeaderFooterIndex);
            }

            @Override
            public void onTabUnselected(TabLayout.Tab tab) {}

            @Override
            public void onTabReselected(TabLayout.Tab tab) {}
        });
    }

    private void initPageImage() {
        CPDFPage currentPage = document.pageAtIndex(0);

        RectF pageSize = document.getPageSize(0);
        float pageW = pageSize.right - pageSize.left;
        float pageH = pageSize.bottom - pageSize.top;

        if (pageW / pageH >  ((float) pageContainer.getWidth() / (float) pageContainer.getHeight())) {
            sizeConvertRate = pageContainer.getWidth() / pageW;

            imageHeight = (int) (sizeConvertRate * pageH);
            imageWidth = pageContainer.getWidth();

            pageImage.setMaxHeight(imageHeight);
            pageImage.setMaxWidth(imageWidth);
        } else {
            sizeConvertRate = pageContainer.getHeight() / pageH;

            imageHeight = pageContainer.getHeight();
            imageWidth = (int) (sizeConvertRate * pageW);

            pageImage.setMaxWidth(imageWidth);
        }

        currentPageHeight = imageHeight;
        currentPageWidth = imageWidth;
        Bitmap pageBitmap = Bitmap.createBitmap(imageWidth, imageHeight, Bitmap.Config.ARGB_4444);

        currentPage.renderPage(pageBitmap, 0, 0, imageWidth, imageHeight, 0xFFFFFFFF, 255, 0, true, true);
        DocumentHandle.refreshPageImage(pageBitmap, pageImage);
    }

    private void initHeaderFooterViews() {
        int[] IDs = new int[] {R.id.left_header_view, R.id.center_header_view, R.id.right_header_view, R.id.left_footer_view, R.id.center_footer_view, R.id.right_footer_view};

        for (int i = 0; i < 6; i++) {
            headerFooterViews[i] = findViewById(IDs[i]);
            headerFooterViews[i].setText(HeaderFooterHandle.getSettingText(type, i));

            int current = i;
            headerFooterViews[i].post(() -> headerFooterViews[current].setTextSize(TypedValue.COMPLEX_UNIT_PX, sizeConvertRate * HeaderFooterHandle.getSettingFontSize(type, current)));
        }
    }

    public void setHeaderFooterViewContent(int index, String content) {
        headerFooterViews[index].setText(content);
    }

    public void setHeaderFooterViewsFontName(int index, String fontName) {
        switch (fontName) {
            case "Roboto":
                headerFooterViews[index].setTypeface(Typeface.DEFAULT);
                break;
            case "Sans":
                headerFooterViews[index].setTypeface(Typeface.SANS_SERIF);
                break;
            case "Monospace":
                headerFooterViews[index].setTypeface(Typeface.MONOSPACE);
                break;
        }
    }

    public void setHeaderFooterViewsSize(int index, float size) {
        headerFooterViews[index].setTextSize(TypedValue.COMPLEX_UNIT_PX, sizeConvertRate * size);
    }

    public void setHeaderFooterViewsColor(int index, int color) {
        headerFooterViews[index].setTextColor(color);
    }

    public void setHeaderFooterViewsMargin(RectF margin) {
        RectF viewMargin = ConvertMapper.convertMMtoViewUnitRectF(margin, sizeConvertRate);

        headerFooterViews[0].setPadding((int) viewMargin.left, (int) viewMargin.top, 0, 0);
        headerFooterViews[1].setPadding(0, (int) viewMargin.top, 0, 0);
        headerFooterViews[2].setPadding(0, (int) viewMargin.top, (int) viewMargin.right, 0);

        headerFooterViews[3].setPadding((int) viewMargin.left, 0, 0, (int) viewMargin.bottom);
        headerFooterViews[4].setPadding(0, 0, 0, (int) viewMargin.bottom);
        headerFooterViews[5].setPadding(0, 0, (int) viewMargin.right, (int) viewMargin.bottom);
    }

    public void reloadDocument() throws CPDFDocumentException {
        if (document.hasChanges()) {
            boolean res;
            res = document.save();
        }
        document.close();

        File PDFFile = new File("/storage/emulated/0/TestPDFs/test.pdf");
        Uri PDFUri = Uri.fromFile(PDFFile);

        pageCount = DocumentHandle.reloadDocument(PDFFile, PDFUri, document);
        HeaderFooterHandle.setSettingPages(0, "0-" + (pageCount - 1));
        HeaderFooterHandle.setSettingPages(1, "0-" + (pageCount - 1));

        CPDFPage currentPage = document.pageAtIndex(0);

        Bitmap pageBitmap = Bitmap.createBitmap(currentPageWidth, currentPageHeight, Bitmap.Config.ARGB_4444);
        currentPage.renderPage(pageBitmap, 0, 0, currentPageWidth, currentPageHeight, 0xFFFFFFFF, 255, 0, true, true);
        DocumentHandle.refreshPageImage(pageBitmap, pageImage);
    }

    public void showGlobalSetting(MenuItem menuItem) {
        DialogManager.showGlobalSettingDialog(this, type);
    }

    public void finishEditChange(MenuItem menuItem) throws CPDFDocumentException {
        if (type == 0)  {
            CPDFHeaderFooter headerFooter = document.getHeaderFooter();
            HeaderFooterHandle.updateCommonType(headerFooter);

        } else {
            CPDFBates bates = document.getBates();
            HeaderFooterHandle.updateBatesType(bates);
        }
        reloadDocument();
    }

    public int getCurrentIndex() {
        return currentHeaderFooterIndex;
    }

    public void changeType(int type) {
        this.type = type;
        setHeaderFooterViewsMargin(HeaderFooterHandle.getSettingMargin(type));
    }

    public int getPageCount() { return pageCount; }
}