package com.compdfkit.pdfviewer.adapter;

import android.annotation.SuppressLint;
import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.compdfkit.core.common.CPDFDocumentException;
import com.compdfkit.pdfviewer.activities.WatermarkActivity;
import com.compdfkit.pdfviewer.R;
import com.compdfkit.pdfviewer.entity.WatermarkSetting;
import com.compdfkit.pdfviewer.fragments.WatermarkConsoleFragment;

import java.util.ArrayList;

public class WatermarkListAdapter extends RecyclerView.Adapter<WatermarkListAdapter.WatermarkItemHolder> {
    private final WatermarkConsoleFragment console;

    private final Context context;
    private View watermarkListView;

    private final ArrayList<WatermarkSetting> settings;

    public WatermarkListAdapter(Context context, WatermarkConsoleFragment console, ArrayList<WatermarkSetting> settings) {
        this.context = context;
        this.console = console;
        this.settings = settings;
    }

    @NonNull
    @Override
    public WatermarkItemHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        watermarkListView = LayoutInflater.from(parent.getContext()).inflate(R.layout.watermark_item_layout, parent, false);

        return new WatermarkItemHolder(watermarkListView);
    }

    @SuppressLint("UseCompatLoadingForDrawables")
    @Override
    public void onBindViewHolder(@NonNull WatermarkItemHolder holder, int position) {

        WatermarkSetting currentSetting = settings.get(position);

        holder.hintIcon.setImageDrawable(context.getResources().getDrawable(currentSetting.getType() == 0 ? R.drawable.ic_baseline_text_type_24 : R.drawable.ic_baseline_image_type_24, context.getTheme()));
        holder.hintText.setText(currentSetting.getType() == 0 ? currentSetting.getText() : "Image\nwatermark");

        holder.itemView.setOnClickListener(view -> {
            WatermarkSetting setting = settings.get(holder.getAdapterPosition());
            try {
                console.resetWatermarkFromSetting(holder.getAdapterPosition(), setting);
            } catch (CPDFDocumentException e) {
                e.printStackTrace();
            }
        });

        holder.deleteView.setOnClickListener(view -> {

            WatermarkActivity activity = (WatermarkActivity) console.getActivity();
            assert activity != null;

            try {
                activity.deleteWatermark(holder.getAdapterPosition());
            } catch (CPDFDocumentException e) {
                e.printStackTrace();
            }

            deleteSettingFromList(holder.getAdapterPosition());
        });
    }

    @Override
    public int getItemCount() {
        return settings.size();
    }

    public void updateSettingList(int index, WatermarkSetting setting) {
        settings.set(index, setting);
        notifyItemChanged(index);
    }

    public void addSettingIntoList(WatermarkSetting setting) {
        settings.add(setting);
        notifyItemChanged(settings.size());
    }

    public void deleteSettingFromList(int index) {
        settings.remove(index);
        notifyItemRemoved(index);
    }

    public static class WatermarkItemHolder extends RecyclerView.ViewHolder {
        int deleteViewOffset;

        ImageView hintIcon;
        TextView hintText;
        View itemView;
        View deleteView;

        public WatermarkItemHolder(@NonNull View itemView) {
            super(itemView);
            hintIcon = itemView.findViewById(R.id.watermark_type_icon);
            hintText = itemView.findViewById(R.id.watermark_content);
            this.itemView = itemView.findViewById(R.id.item_view);
            deleteView = itemView.findViewById(R.id.delete_view);

            deleteView.post(() -> deleteViewOffset = deleteView.getWidth());
        }

        public int getDeleteViewOffset() {
            return deleteViewOffset;
        }
    }
}
