package com.compdfkit.pdfviewer.customview.picker;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Color;
import android.util.AttributeSet;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.constraintlayout.widget.ConstraintLayout;

import com.compdfkit.pdfviewer.R;
import com.compdfkit.pdfviewer.tools.BackgroundHandle;

public class BackgroundColorPicker extends ConstraintLayout {
    private View currentView;
    private LayoutInflater inflater;
    private Context currentContext;

    private int backgroundWidth;
    private int backgroundHeight;

    private ImageView backgroundView;

    private final int[] colorPickers = { R.id.color_selected, R.id.color_1, R.id.color_2, R.id.color_3, R.id.color_4, R.id.color_5, R.id.color_6, R.id.color_7 };
    private final int[] colors = { 0xFFFFFFFF, 0xFFFF0000, 0xFFFF6600, 0xFFFFDD00, 0xFF00FF33, 0xFF00FFFF, 0xFF000000, 0xFF8800FF };

    public BackgroundColorPicker(@NonNull Context context) {
        this(context, null);
    }

    public BackgroundColorPicker(@NonNull Context context, @Nullable AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public BackgroundColorPicker(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        this(context, attrs, defStyleAttr, 0);
    }

    public BackgroundColorPicker(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr, int defStyleRes) {
        super(context, attrs, defStyleAttr, defStyleRes);
        init(context);
    }

    private void init(Context context) {
        currentContext = context;
        inflater = (LayoutInflater) currentContext.getSystemService(Context.LAYOUT_INFLATER_SERVICE);

        currentView = inflater.inflate(R.layout.color_picker, this, true);

        initColorPicker(currentView);
    }

    private void initColorPicker(View currentView) {
        for (int i = 0; i < 8; i++) {
            ImageView tempColor = currentView.findViewById(colorPickers[i]);

            int tempI = i;
            tempColor.setOnClickListener(view -> {
                BackgroundHandle.setSettingColor(colors[tempI]);
                BackgroundHandle.setSettingImage(null);

                Bitmap backgroundColor = Bitmap.createBitmap(backgroundWidth, backgroundHeight, Bitmap.Config.ARGB_4444);
                backgroundColor.eraseColor(colors[tempI]);
                backgroundView.setImageBitmap(backgroundColor);
                backgroundView.setAlpha(BackgroundHandle.getBackgroundSetting().getOpacity());
            });
        }
    }

    public void setBackgroundView(ImageView backgroundView, int backgroundWidth, int backgroundHeight) {
        this.backgroundView = backgroundView;
        this.backgroundWidth = backgroundWidth;
        this.backgroundHeight = backgroundHeight;
    }
}
