package com.compdfkit.pdfviewer.customview.watermarkview;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Matrix;
import android.util.AttributeSet;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.ImageButton;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.constraintlayout.widget.ConstraintLayout;

import com.compdfkit.pdfviewer.entity.WatermarkSetting;

public class WatermarkView extends ConstraintLayout {
    protected Context currentContext;
    protected LayoutInflater inflater;
    protected View currentView;
    protected ImageButton rollingHandle;

    protected float rotationRadian = 0;
    protected float sizeConvertRate = 0;

    protected float viewXOffset = 0;
    protected float viewYOffset = 0;

    protected float PDFXOffset = 0;
    protected float PDFYOffset = 0;

    protected float scale = 1;

    public static int DRAG_EVENT = 0;
    public static int ROLL_EVENT = 1;

    //TODO:Try to change the event select.
    protected int currentEvent = 0;

    public WatermarkView(@NonNull Context context) {
        this(context, null);
    }

    public WatermarkView(@NonNull Context context, @Nullable AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public WatermarkView(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        this(context, attrs, defStyleAttr, 0);
    }

    public WatermarkView(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr, int defStyleRes) {
        super(context, attrs, defStyleAttr, defStyleRes);
    }

    public void setRollingHandle() {}

    public void setRotationRadian(float rotationRadian) {
        this.rotationRadian = rotationRadian;
    }

    public float getRotationRadian() {
        return rotationRadian;
    }

    public void setWatermarkOpacity(float opacity) {}

    public void setWatermarkScale(float scale) {}

    public void setTextWatermarkColor(int color) {}

    public WatermarkSetting completeSetting(WatermarkSetting setting) {
        return setting;
    }

    public int getCurrentEvent() {
        return currentEvent;
    }

    public float getViewXOffset() {
        return viewXOffset;
    }

    public void setViewXOffset(float viewXOffset) {
        this.viewXOffset = viewXOffset;
    }

    public float getViewYOffset() {
        return viewYOffset;
    }

    public void setViewYOffset(float viewYOffset) {
        this.viewYOffset = viewYOffset;
    }

    public float getPDFXOffset() {
        return PDFXOffset;
    }

    public void setPDFXOffset(float PDFXOffset) {
        this.PDFXOffset = PDFXOffset;
    }

    public float getPDFYOffset() {
        return PDFYOffset;
    }

    public void setPDFYOffset(float PDFYOffset) {
        this.PDFYOffset = PDFYOffset;
    }

    public float getScale() {
        return scale;
    }

    public void setScale(float scale) {
        this.scale = scale;
    }

    public float getSizeConvertRate() {
        return sizeConvertRate;
    }

    public void setSizeConvertRate(float sizeConvertRate) {
        this.sizeConvertRate = sizeConvertRate;
    }

    public Bitmap generateFullScreenBitmap(Bitmap srcBitmap, int width, int height, int hSpace, int vSpace, int xOffset, int yOffset, float rotation) {
        int srcWidth = srcBitmap.getWidth();
        int srcHeight = srcBitmap.getHeight();

        int longer = Math.max(width, height);

        int xTimes = longer * 2 / (srcWidth + hSpace);
        int yTimes = longer * 2 / (srcHeight + vSpace);

        int centerX = width / 2;
        int centerY = height / 2;

        int centerXOffset = centerX % (srcWidth + hSpace) - (srcWidth + hSpace) / 2;
        int centerYOffset = centerY % (srcHeight + vSpace) - (srcHeight + vSpace) / 2;

        int originXOffset = xOffset;
        int originYOffset = yOffset;

        xOffset = xOffset % (srcWidth + hSpace);
        yOffset = yOffset % (srcHeight + vSpace);

        Matrix matrix = new Matrix();

        Bitmap bitmap = Bitmap.createBitmap(longer * 2, longer * 2, Bitmap.Config.ARGB_4444);
        Canvas canvas = new Canvas(bitmap);

        int paintOffsetX = (srcWidth + hSpace) - (((bitmap.getWidth() - width) / 2) % (srcWidth + hSpace));
        int paintOffsetY = (srcHeight + vSpace) - (((bitmap.getHeight() - height) / 2) % (srcHeight + vSpace));

        for (int i = -1; i < xTimes + 1; i++) {
            for(int j = -1; j < yTimes + 1; j++) {
                int left = i * (srcWidth + hSpace) + centerXOffset + hSpace / 2  - paintOffsetX;
                int top = j * (srcHeight + vSpace) + centerYOffset + vSpace / 2 - paintOffsetY;

                canvas.drawBitmap(srcBitmap, left, top, null);
            }
        }

        matrix.setTranslate(longer, longer);
        matrix.postTranslate(xOffset, -yOffset);
        matrix.postRotate(rotation, 2 * longer + originXOffset, 2 *longer - originYOffset);

        Bitmap testBitmap = Bitmap.createBitmap(bitmap.getWidth() * 2, bitmap.getHeight() * 2, Bitmap.Config.ARGB_4444);
        Canvas testCanvas = new Canvas(testBitmap);
        testCanvas.drawBitmap(bitmap, matrix, null);

        return Bitmap.createBitmap(testBitmap, (testBitmap.getWidth() - width) / 2, (testBitmap.getHeight() - height) / 2, width, height);
    }

    public Bitmap getFullScreenBitmap(int width, int height, int hSpace, int vSpace, int xOffset, int yOffset, float rotation) {
        return generateFullScreenBitmap(null, width, height, hSpace, vSpace, xOffset, yOffset, rotation);
    }
}
