package com.compdfkit.pdfviewer.fragments;

import static android.app.Activity.RESULT_OK;

import android.Manifest;
import android.annotation.SuppressLint;
import android.content.ContentUris;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.database.Cursor;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;

import androidx.annotation.Nullable;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;
import androidx.fragment.app.Fragment;

import android.provider.DocumentsContract;
import android.provider.MediaStore;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.TextView;

import com.compdfkit.pdfviewer.R;
import com.compdfkit.pdfviewer.activities.BackgroundActivity;
import com.compdfkit.pdfviewer.activities.HeaderFooterActivity;
import com.compdfkit.pdfviewer.customview.picker.BackgroundColorPicker;
import com.compdfkit.pdfviewer.customview.picker.ImagePicker;
import com.compdfkit.pdfviewer.customview.watermarkview.ImageWatermarkView;
import com.compdfkit.pdfviewer.tools.BackgroundHandle;

public class BackgroundConsoleFragment extends Fragment {

    private BackgroundActivity activity;

    private ImageView backgroundLayer;
    private BackgroundColorPicker colorPicker;
    private ImagePicker imagePicker;

    private TextView imageSelectPathView;

    private static final String ARG_PARAM1 = "param1";
    private static final String ARG_PARAM2 = "param2";

    // TODO: Rename and change types of parameters
    private String mParam1;
    private String mParam2;

    public BackgroundConsoleFragment() {
    }

    public static BackgroundConsoleFragment newInstance(String param1, String param2) {
        BackgroundConsoleFragment fragment = new BackgroundConsoleFragment();
        Bundle args = new Bundle();
        args.putString(ARG_PARAM1, param1);
        args.putString(ARG_PARAM2, param2);
        fragment.setArguments(args);
        return fragment;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        if (getArguments() != null) {
            mParam1 = getArguments().getString(ARG_PARAM1);
            mParam2 = getArguments().getString(ARG_PARAM2);
        }
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        activity = (BackgroundActivity) getActivity();
        View returnView = inflater.inflate(R.layout.fragment_background_console, container, false);
        initView(returnView);

        return returnView;
    }

    private void initView(View returnView) {
        colorPicker = returnView.findViewById(R.id.color_picker);
        colorPicker.post(() -> colorPicker.setBackgroundView(activity.getBackgroundLayer(), activity.getCurrentPageWidth(), activity.getCurrentPageHeight()));

        backgroundLayer = activity.getBackgroundLayer();
        imagePicker = returnView.findViewById(R.id.image_picker);
        Button imageSelectBtn = imagePicker.getImageSelectBtn();
        imageSelectPathView = imagePicker.getImageSelectPathView();
        imageSelectBtn.setOnClickListener(v -> {
            if (ContextCompat.checkSelfPermission(getActivity(),
                    Manifest.permission.WRITE_EXTERNAL_STORAGE) != PackageManager.PERMISSION_GRANTED) {
                ActivityCompat.requestPermissions(getActivity(), new String[]{Manifest.permission.WRITE_EXTERNAL_STORAGE}, 101);
            } else {
                //打开相册
                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
                    Intent intent = new Intent(MediaStore.ACTION_PICK_IMAGES);
                    startActivityForResult(intent, 102);
                } else {
                    Intent intent = new Intent(Intent.ACTION_GET_CONTENT);
                    //Intent.ACTION_GET_CONTENT = "android.intent.action.GET_CONTENT"
                    intent.setType("image/*");
                    startActivityForResult(intent, 102); // 打开相册
                }
            }
        });
    }

    @Override
    //TODO:Replace the deprecated method.
    public void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);

        if (resultCode == RESULT_OK) {
            handleImageOnKitKat(data);
        }
    }

    private void handleImageOnKitKat(Intent data) {
        String imagePath = null;
        Uri uri = data.getData();
        if (DocumentsContract.isDocumentUri(getActivity(), uri)) {
            String docId = DocumentsContract.getDocumentId(uri);
            if ("com.android.providers.media.documents".equals(uri.getAuthority())) {
                String id = docId.split(":")[1];
                String selection = MediaStore.Images.Media._ID + "=" + id;
                imagePath = getImagePath(MediaStore.Images.Media.EXTERNAL_CONTENT_URI, selection);
            } else if ("com.android.providers.downloads.documents".equals(uri.getAuthority())) {
                Uri contentUri = ContentUris.withAppendedId(Uri.parse("content: //downloads/public_downloads"), Long.valueOf(docId));
                imagePath = getImagePath(contentUri, null);
            }
        } else if ("content".equalsIgnoreCase(uri.getScheme())) {
            imagePath = getImagePath(uri, null);
        } else if ("file".equalsIgnoreCase(uri.getScheme())) {
            imagePath = uri.getPath();
        }
        imageSelectPathView.setText(imagePath);

        Bitmap backgroundImage = BitmapFactory.decodeFile(imagePath);
        BackgroundHandle.setSettingImage(backgroundImage);
        backgroundLayer.setImageBitmap(backgroundImage);
    }

    @SuppressLint("Range")
    private String getImagePath(Uri uri, String selection) {
        String path = null;

        Cursor cursor = getActivity().getContentResolver().query(uri, null, selection, null, null);
        if (cursor != null) {
            if (cursor.moveToFirst()) {
                path = cursor.getString(cursor.getColumnIndex(MediaStore.Images.Media.DATA));
            }
            cursor.close();
        }
        return path;
    }
}