package com.compdfkit.pdfviewer.tools;

import android.annotation.SuppressLint;
import android.content.Context;
import android.content.DialogInterface;
import android.graphics.RectF;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.AdapterView;
import android.widget.AutoCompleteTextView;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.widget.AppCompatAutoCompleteTextView;
import androidx.appcompat.widget.AppCompatEditText;
import androidx.appcompat.widget.AppCompatRadioButton;
import androidx.constraintlayout.widget.ConstraintLayout;

import com.compdfkit.pdfviewer.R;
import com.compdfkit.pdfviewer.activities.BackgroundActivity;
import com.compdfkit.pdfviewer.activities.HeaderFooterActivity;
import com.compdfkit.pdfviewer.customview.picker.FontColorPicker;
import com.compdfkit.pdfviewer.entity.BackgroundSetting;
import com.google.android.material.dialog.MaterialAlertDialogBuilder;
import com.google.android.material.slider.Slider;
import com.google.android.material.textfield.TextInputEditText;

import java.util.Objects;

public class DialogManager {
    @SuppressLint("SetTextI18n")
    public static void showBackgroundSettingDialog(Context context) {
        BackgroundActivity activity = (BackgroundActivity) context;

        LayoutInflater inflater = LayoutInflater.from(context);
        final ConstraintLayout backgroundSettingLayout = (ConstraintLayout) inflater.inflate(R.layout.background_setting_panel, null);
        final Button[] positiveButton = new Button[1];
        boolean isCustomRange = true;

        BackgroundSetting currentSetting = BackgroundHandle.getBackgroundSetting();

        TextInputEditText xOffset = backgroundSettingLayout.findViewById(R.id.x_offset);
        xOffset.setText(String.valueOf(currentSetting.getxOffset()));
        TextInputEditText yOffset = backgroundSettingLayout.findViewById(R.id.y_offset);
        yOffset.setText(String.valueOf(currentSetting.getyOffset()));

        Slider opacitySlider = backgroundSettingLayout.findViewById(R.id.opacity_slider);
        opacitySlider.setValue(currentSetting.getOpacity() * 100);
        Slider sizeSlider = backgroundSettingLayout.findViewById(R.id.size_slider);
        sizeSlider.setValue(currentSetting.getScale() * 100);
        Slider rotationSlider = backgroundSettingLayout.findViewById(R.id.rotation_slider);
        rotationSlider.setValue((float) Math.toDegrees(currentSetting.getRotation()));

        TextView opacityText = backgroundSettingLayout.findViewById(R.id.opacity_text);
        opacityText.setText((int) opacitySlider.getValue() + "%");
        TextView sizeText = backgroundSettingLayout.findViewById(R.id.size_text);
        sizeText.setText((int) sizeSlider.getValue() + "%");
        TextView rotationText = backgroundSettingLayout.findViewById(R.id.rotation_text);
        rotationText.setText((int) rotationSlider.getValue() + "°");

        ImageButton pageRangeBtn = backgroundSettingLayout.findViewById(R.id.page_range_button);
        TextView pageRangeMsg = backgroundSettingLayout.findViewById(R.id.page_range_msg);
        isCustomRange = !BackgroundHandle.getSettingPages().equals("0-" + (activity.getPageCount() - 1));

        boolean finalIsCustomRange = isCustomRange;
        pageRangeBtn.setOnClickListener((view) -> showPageRangeDialog(context, 3, finalIsCustomRange, BackgroundHandle.getSettingPages(), pageRangeMsg, activity.getPageCount()));

        opacitySlider.addOnChangeListener((slider, value, fromUser) -> opacityText.setText((int) value + "%"));
        sizeSlider.addOnChangeListener((slider, value, fromUser) -> sizeText.setText((int) value + "%"));
        rotationSlider.addOnChangeListener((slider, value, fromUser) -> rotationText.setText((int) value + "°"));

        MaterialAlertDialogBuilder inputDialog = new MaterialAlertDialogBuilder(context);
        inputDialog.setTitle(R.string.background_setting).setView(backgroundSettingLayout);
        inputDialog.setPositiveButton(R.string.done, (dialogInterface, i) -> {
            BackgroundHandle.setSettingMsg(
                    Float.parseFloat(xOffset.getText().toString()),
                    Float.parseFloat(yOffset.getText().toString()),
                    opacitySlider.getValue() / 100,
                    sizeSlider.getValue() / 100,
                    (float) Math.toRadians(rotationSlider.getValue()));

            activity.setBackgroundLayerAttr(
                    Float.parseFloat(xOffset.getText().toString()),
                    Float.parseFloat(yOffset.getText().toString()),
                    opacitySlider.getValue() / 100,
                    sizeSlider.getValue() / 100,
                    -rotationSlider.getValue());
        });

        inputDialog.setNegativeButton(R.string.cancel, (dialogInterface, i) -> {});

        AlertDialog dialog = inputDialog.create();
        dialog.show();
        positiveButton[0] = dialog.getButton(DialogInterface.BUTTON_POSITIVE);
    }

    public static void showGlobalSettingDialog(Context context, int type) {
        LayoutInflater inflater = LayoutInflater.from(context);
        final ConstraintLayout globalSettingLayout = (ConstraintLayout) inflater.inflate(R.layout.global_setting_panel, null);
        final Button[] positiveButton = new Button[1];

        HeaderFooterActivity activity = (HeaderFooterActivity) context;
        boolean isCustomRange;

        TextInputEditText pageOffsetEdit = globalSettingLayout.findViewById(R.id.start_page);
        pageOffsetEdit.setText(String.valueOf(HeaderFooterHandle.getSettingPageOffset(type)));

        RectF currentMargin = HeaderFooterHandle.getSettingMargin(type);

        TextInputEditText topMargin = globalSettingLayout.findViewById(R.id.top_margin);
        topMargin.setText(String.valueOf(currentMargin.top));
        TextInputEditText bottomMargin = globalSettingLayout.findViewById(R.id.bottom_margin);
        bottomMargin.setText(String.valueOf(currentMargin.bottom));
        TextInputEditText leftMargin = globalSettingLayout.findViewById(R.id.left_margin);
        leftMargin.setText(String.valueOf(currentMargin.left));
        TextInputEditText rightMargin = globalSettingLayout.findViewById(R.id.right_margin);
        rightMargin.setText(String.valueOf(currentMargin.right));

        ImageButton pageRangeBtn = globalSettingLayout.findViewById(R.id.page_range_button);
        TextView pageRangeMsg = globalSettingLayout.findViewById(R.id.page_range_msg);
        isCustomRange = !HeaderFooterHandle.getSettingPages(type).equals("0-" + (activity.getPageCount() - 1));

        if (isCustomRange)
            pageRangeMsg.setText(HeaderFooterHandle.getSettingPages(type));

        boolean finalIsCustomRange = isCustomRange;
        pageRangeBtn.setOnClickListener((view) -> showPageRangeDialog(context, type, finalIsCustomRange, HeaderFooterHandle.getSettingPages(type), pageRangeMsg, activity.getPageCount()));

        MaterialAlertDialogBuilder inputDialog = new MaterialAlertDialogBuilder(context);
        inputDialog.setTitle(R.string.global_header_footer_setting).setView(globalSettingLayout);
        inputDialog.setPositiveButton(R.string.done, (dialogInterface, i) -> {
            float top = Float.parseFloat(topMargin.getText().toString());
            float bottom = Float.parseFloat(bottomMargin.getText().toString());
            float left = Float.parseFloat(leftMargin.getText().toString());
            float right = Float.parseFloat(rightMargin.getText().toString());

            RectF margin = new RectF(left, top, right, bottom);
            HeaderFooterHandle.setSettingMargin(type, margin);

            activity.setHeaderFooterViewsMargin(margin);

            int pageOffset = Integer.parseInt(pageOffsetEdit.getText().toString());
            HeaderFooterHandle.setSettingPageOffset(type, pageOffset);
        });

        inputDialog.setNegativeButton(R.string.cancel, (dialogInterface, i) -> {});

        AlertDialog dialog = inputDialog.create();
        dialog.show();
        positiveButton[0] = dialog.getButton(DialogInterface.BUTTON_POSITIVE);
    }

    public static void showCommonSettingDialog(Context context, int index) {
        String[] dateFormats = context.getResources().getStringArray(R.array.date_format);
        String[] pageFormats = context.getResources().getStringArray(R.array.page_format);

        final int[] tempFormatIndex = { HeaderFooterHandle.getIndexDateFormat(index), HeaderFooterHandle.getIndexPageFormat(index) };

        LayoutInflater inflater = LayoutInflater.from(context);
        final ConstraintLayout commonSettingLayout = (ConstraintLayout) inflater.inflate(R.layout.common_setting_panel, null);
        final Button[] positiveButton = new Button[1];

        TextView fontColorText = commonSettingLayout.findViewById(R.id.font_color_text);
        FontColorPicker fontColorPicker = commonSettingLayout.findViewById(R.id.color_picker);
        fontColorPicker.setTextView(fontColorText);

        AutoCompleteTextView fontPicker = commonSettingLayout.findViewById(R.id.font_picker);
        fontPicker.setText(HeaderFooterHandle.getSettingFontName(0, index), false);

        AutoCompleteTextView fontSizePicker = commonSettingLayout.findViewById(R.id.font_size_picker);
        fontSizePicker.setText(String.valueOf(HeaderFooterHandle.getSettingFontSize(0, index)), false);

        AutoCompleteTextView dateFormatPicker = commonSettingLayout.findViewById(R.id.date_format_picker);
        dateFormatPicker.setText(dateFormats[tempFormatIndex[0]], false);
        dateFormatPicker.setOnItemClickListener((parent, view, position, id) -> tempFormatIndex[0] = position);

        AutoCompleteTextView pageFormatPicker = commonSettingLayout.findViewById(R.id.page_format_picker);
        pageFormatPicker.setText(pageFormats[tempFormatIndex[1]], false);
        pageFormatPicker.setOnItemClickListener((parent, view, position, id) -> tempFormatIndex[1] = position);

        EditText startPagePicker = commonSettingLayout.findViewById(R.id.start_page_picker);
        startPagePicker.setText(String.valueOf(HeaderFooterHandle.getIndexStartPageNumber(index)));

        MaterialAlertDialogBuilder inputDialog = new MaterialAlertDialogBuilder(context);
        inputDialog.setTitle(R.string.headerfooter_setting).setView(commonSettingLayout);

        inputDialog.setPositiveButton(R.string.done, (dialogInterface, i) -> {
            HeaderFooterHandle.setSettingColor(0, index, fontColorText.getCurrentTextColor());
            HeaderFooterHandle.setSettingFontName(0, index, fontPicker.getText().toString());
            HeaderFooterHandle.setSettingFontSize(0, index, Float.parseFloat(fontSizePicker.getText().toString()));

            HeaderFooterActivity activity = (HeaderFooterActivity) context;
            activity.setHeaderFooterViewsFontName(index, fontPicker.getText().toString());
            activity.setHeaderFooterViewsSize(index, Float.parseFloat(fontSizePicker.getText().toString()));
            activity.setHeaderFooterViewsColor(index, fontColorText.getCurrentTextColor());

            HeaderFooterHandle.setIndexDateFormat(index, tempFormatIndex[0]);
            HeaderFooterHandle.setIndexPageFormat(index, tempFormatIndex[1]);
            HeaderFooterHandle.setIndexStartPageNumber(index, Integer.parseInt(startPagePicker.getText().toString()));
        });

        inputDialog.setNegativeButton(R.string.cancel, (dialogInterface, i) -> {});

        AlertDialog dialog = inputDialog.create();
        dialog.show();
        positiveButton[0] = dialog.getButton(DialogInterface.BUTTON_POSITIVE);
    }

    public static void showBatesSettingDialog(Context context, int index) {
        LayoutInflater inflater = LayoutInflater.from(context);
        final ConstraintLayout batesSettingLayout = (ConstraintLayout) inflater.inflate(R.layout.bates_setting_panel, null);
        final Button[] positiveButton = new Button[1];

        TextView fontColorText = batesSettingLayout.findViewById(R.id.font_color_text);
        FontColorPicker fontColorPicker = batesSettingLayout.findViewById(R.id.color_picker);
        fontColorPicker.setTextView(fontColorText);

        AutoCompleteTextView fontPicker = batesSettingLayout.findViewById(R.id.font_picker);
        fontPicker.setText(HeaderFooterHandle.getSettingFontName(1, index), false);

        AutoCompleteTextView fontSizePicker = batesSettingLayout.findViewById(R.id.font_size_picker);
        fontSizePicker.setText(String.valueOf(HeaderFooterHandle.getSettingFontSize(1, index)), false);

        TextInputEditText prefixEdit = batesSettingLayout.findViewById(R.id.prefix);
        prefixEdit.setText(HeaderFooterHandle.getIndexPrefix(index));
        TextInputEditText suffixEdit = batesSettingLayout.findViewById(R.id.suffix);
        suffixEdit.setText(HeaderFooterHandle.getIndexSuffix(index));
        TextInputEditText digitsEdit = batesSettingLayout.findViewById(R.id.number);
        digitsEdit.setText(String.valueOf(HeaderFooterHandle.getIndexDigits(index)));

        MaterialAlertDialogBuilder inputDialog = new MaterialAlertDialogBuilder(context);
        inputDialog.setTitle(R.string.global_bates_setting).setView(batesSettingLayout);
        inputDialog.setPositiveButton(R.string.done, (dialogInterface, i) -> {
            HeaderFooterHandle.setSettingColor(1, index, fontColorText.getCurrentTextColor());
            HeaderFooterHandle.setSettingFontName(1, index, fontPicker.getText().toString());
            HeaderFooterHandle.setSettingFontSize(1, index, Float.parseFloat(fontSizePicker.getText().toString()));

            HeaderFooterActivity activity = (HeaderFooterActivity) context;
            activity.setHeaderFooterViewsFontName(index, fontPicker.getText().toString());
            activity.setHeaderFooterViewsSize(index, Float.parseFloat(fontSizePicker.getText().toString()));
            activity.setHeaderFooterViewsColor(index, fontColorText.getCurrentTextColor());

            HeaderFooterHandle.setIndexPrefix(index, prefixEdit.getText().toString());
            HeaderFooterHandle.setIndexSuffix(index, suffixEdit.getText().toString());
            HeaderFooterHandle.setIndexDigits(index, Integer.parseInt(digitsEdit.getText().toString()));
        });

        inputDialog.setNegativeButton(R.string.cancel, (dialogInterface, i) -> {});

        AlertDialog dialog = inputDialog.create();
        dialog.show();
        positiveButton[0] = dialog.getButton(DialogInterface.BUTTON_POSITIVE);
    }

    public static void showPageRangeDialog(Context context, int type, boolean isCustomRange, String pageRange, TextView textView, int pageCount) {
        LayoutInflater inflater = LayoutInflater.from(context);
        final ConstraintLayout rangeSelectLayout = (ConstraintLayout) inflater.inflate(R.layout.range_selector, null);
        final Button[] positiveButton = new Button[1];

        AppCompatRadioButton customRadioBtn = rangeSelectLayout.findViewById(R.id.custom_range);
        customRadioBtn.setChecked(isCustomRange);

        AppCompatEditText rangeEditText = rangeSelectLayout.findViewById(R.id.range_edit);
        rangeEditText.setEnabled(isCustomRange);
        if (isCustomRange)
            rangeEditText.setText(pageRange);

        customRadioBtn.setOnCheckedChangeListener((compoundButton, b) -> {
            rangeEditText.setEnabled(b);
            if (b && Objects.requireNonNull(rangeEditText.getText()).toString().equals(""))
                positiveButton[0].setEnabled(false);
        });

        rangeEditText.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {}

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {}

            @Override
            public void afterTextChanged(Editable editable) {
                positiveButton[0].setEnabled(!rangeEditText.getText().toString().equals(""));
            }
        });

        MaterialAlertDialogBuilder inputDialog = new MaterialAlertDialogBuilder(context);
        inputDialog.setTitle(R.string.setting_range).setView(rangeSelectLayout);

        inputDialog.setPositiveButton(R.string.done, (dialogInterface, i) -> {
            if (customRadioBtn.isChecked()) {
                if (type != 3)
                    HeaderFooterHandle.setSettingPages(type, rangeEditText.getText().toString());
                else
                    BackgroundHandle.setSettingPages(rangeEditText.getText().toString());

                textView.setText(rangeEditText.getText().toString());
            } else {
                if (type != 3)
                    HeaderFooterHandle.setSettingPages(type, "0-" + (pageCount - 1));
                else
                    BackgroundHandle.setSettingPages("0-" + (pageCount - 1));

                textView.setText(R.string.all_pages);
            }
        });

        AlertDialog dialog = inputDialog.create();
        dialog.show();
        positiveButton[0] = dialog.getButton(DialogInterface.BUTTON_POSITIVE);
    }

    @SuppressLint("SetTextI18n")
    public static void showContentAppendDialog(Context context, int currentIndex, EditText contentEdit) {
        String[] dateFormats = context.getResources().getStringArray(R.array.date_format);
        String[] pageFormats = context.getResources().getStringArray(R.array.page_format);

        LayoutInflater inflater = LayoutInflater.from(context);
        final ConstraintLayout contentAppendLayout = (ConstraintLayout) inflater.inflate(R.layout.content_append_selector, null);
        final Button[] positiveButton = new Button[1];

        AppCompatRadioButton addDateRadioBtn = contentAppendLayout.findViewById(R.id.add_date);
        addDateRadioBtn.setChecked(true);

        MaterialAlertDialogBuilder inputDialog = new MaterialAlertDialogBuilder(context);
        inputDialog.setTitle("Select content").setView(contentAppendLayout);
        inputDialog.setPositiveButton(R.string.done, (dialogInterface, i) -> {
            String content = contentEdit.getText().toString();
            if (addDateRadioBtn.isChecked()) {
                contentEdit.setText(content + "<<" + dateFormats[HeaderFooterHandle.getIndexDateFormat(currentIndex)] + ">>");
            } else {
                contentEdit.setText(content + "<<" + pageFormats[HeaderFooterHandle.getIndexPageFormat(currentIndex)] + ">>");
            }
        });

        AlertDialog dialog = inputDialog.create();
        dialog.show();
        positiveButton[0] = dialog.getButton(DialogInterface.BUTTON_POSITIVE);
    }
}
