package com.compdfkit.pdfviewer.tools;

import com.compdfkit.core.watermark.CPDFWatermark;
import com.compdfkit.pdfviewer.entity.WatermarkSetting;

import java.util.ArrayList;

public class WatermarkHandle {
    private static boolean enabled = false;

    private static final ArrayList<WatermarkSetting> settingList = new ArrayList<>();

    public static void updateWatermark(CPDFWatermark watermark, WatermarkSetting setting, float sizeConvertRate) {
        try {
            if (watermark != null && watermark.isValid()) {
                if (setting.getImage() != null)
                    watermark.setImage(setting.getImage(), (int) (setting.getiWidth() * sizeConvertRate), (int) (setting.getiHeight() * sizeConvertRate));
                watermark.setText(setting.getText());
                watermark.setFontName(setting.getFontName());
                watermark.setTextRGBColor(setting.getTextRGBColor());
                watermark.setOpacity(setting.getOpacity());
                watermark.setFontSize(setting.getFontSize());
                watermark.setFront(true);
                watermark.setVertalign(CPDFWatermark.Vertalign.WATERMARK_VERTALIGN_CENTER);
                watermark.setHorizalign(CPDFWatermark.Horizalign.WATERMARK_HORIZALIGN_CENTER);
                watermark.setVertOffset(setting.getVertOffset());
                watermark.setHorizOffset(setting.getHorizOffset());
                watermark.setRotation(setting.getRotation());
                watermark.setScale(setting.getScale());
                watermark.setPages(setting.getPages());
                watermark.setFullScreen(setting.getFullScreen());
                watermark.setHorizontalSpacing(setting.getHorizontalSpacing());
                watermark.setVerticalSpacing(setting.getVerticalSpacing());
                watermark.update();
            }
        } finally {
            if (watermark != null && watermark.isValid()) {
                watermark.release();
            }
        }
    }

    public static WatermarkSetting getSettingFromWatermark(CPDFWatermark watermark) {
        WatermarkSetting setting = new WatermarkSetting();
        setting.setType(watermark.getType() == CPDFWatermark.Type.WATERMARK_TYPE_TEXT ? 0 : 1);

        setting.setImage(watermark.getImage());
        setting.setText(watermark.getText());
        setting.setFontName(watermark.getFontName());
        setting.setTextRGBColor(watermark.getTextRGBColor());
        setting.setOpacity(watermark.getOpacity());
        setting.setFontSize(watermark.getFontSize());
        setting.setFront(true);
        setting.setVertOffset(watermark.getVertOffset());
        setting.setHorizOffset(watermark.getHorizOffset());
        setting.setRotation(watermark.getRotation());
        setting.setScale(watermark.getScale());
        setting.setPages(watermark.getPages());
        setting.setFullScreen(watermark.isFullScreen());
        setting.setHorizontalSpacing(watermark.getHorizontalSpacing());
        setting.setVerticalSpacing(watermark.getVerticalSpacing());

        return setting;
    }

    public static boolean getEnable() {
        return enabled;
    }

    public static void addSettingIntoList(CPDFWatermark watermark) {
        if (!enabled)
            enabled = true;

        settingList.add(getSettingFromWatermark(watermark));
    }

    public static ArrayList<WatermarkSetting> getSettingList() {
        return settingList;
    }

    public static WatermarkSetting getSettingFromList(int index) {
        return settingList.get(index);
    }

    public static void updateSetting(int index, WatermarkSetting setting) {
        settingList.set(index, setting);
    }

    public static void deleteSettingFromList(int index) {
        settingList.remove(index);
    }

    public static void clearSettingList() {
        settingList.clear();
    }
}
